/*
 * DcopRef.cs - Base class for remote dcop objects with callable methods.
 * Is is fairly low-level, don't use it directly.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

namespace Xsharp.Dcop
{

using System;
using Xsharp;

// TODO: Split this into factory and simplier DcopRef
public class DcopRef : ICloneable
{

	private string app = null;
	private string obj = null;
	private string type = null; // We do not use type for now. I'm unsure if we should.

	private DcopClient client;
	private bool initialised = false;

	public DcopRef()
	{
		// TODO: Add some code there?
	}

	/// This meant to be called by QDataStream. You should use DcopRef() &amp; Discover*()
	public DcopRef(string app, string obj)
	{
		this.app = app;
		this.obj = obj;
	}

	public void DiscoverApplication(string app, bool sessioned, bool createIfNotExists)
	{
		// Discovers object
		string[] apps;
		string[] bits;
		if(client == null)
		{
			DiscoverClient();
		}
		apps = client.registeredApplications();
		for(int i = 0; i < apps.Length; i++)
		{
			if(sessioned)
			{
				bits = apps[i].Split(new char[]{'-'});
				if( (String.Join("-", bits, 0, bits.Length - 1) == app) &&
					(Int32.Parse(bits[bits.Length - 1]) > 0) )
				{
					this.app = apps[i];
					return; // Will discover first found
				}
			}
			else
			{
				if(apps[i] == app)
				{
					this.app = apps[i];
					return;
				}
			}
		}
/*		if(createIfNotExist)
		{
			try
			{
				app = ((ServiceResult)client.Call("klauncher", "klauncher", "serviceResult start_service_by_desktop_name(QString,QStringList)")).result;
				return true;
			}
			
				
		}*/
		throw new DcopNamingException("Failed to discover application");
	}

	public void Initialise()
	{
		if(client == null)
		{
			DiscoverClient();
		}
		if(app == null)
		{
			throw new DcopConnectionException("`app' was not initialised");
		}
		if(obj == null)
		{
			this.obj = ""; // Just use default one
		}
		initialised = true;
		string[] inter = interfaces();
		for(int i = 0; i < inter.Length; i++)
		{
			if(inter[i] == "DCOPObject")
			{
				return;
			}
		}
		initialised = false;
		throw new DcopNamingException("Not an object or naming fault");
	}

	public Object Call(string fun, params Object[] args)
	{
		if(!initialised)
		{
			throw new DcopConnectionException("Attempt to do Call()s on non-initialised Dcop reference");
		}
		return client.Call(app, obj, fun, args);
	}

	public Object Clone()
	{
		DcopRef res = new DcopRef();
		res.Obj = obj;
		res.Client = client;
		if(!initialised)
		{
			res.App = app;
		}
		else
		{
			res.DiscoverApplication(app, false, false);
			res.Initialise();
		}
		return res;
	}

	protected void DiscoverClient()
	{
		if(DcopClient.MainClient == null)
		{
			if(Application.Primary != null)
			DcopClient.MainClient = new DcopClient(Application.Primary.Display, null);
		}
		this.client = DcopClient.MainClient;
	}

	[DcopCall("QCStringList objects()")]
	public string[] objects()
	{
		if(app == null)
		{
			throw new DcopConnectionException("`app' is null");
		}
		if(client == null)
		{
			DiscoverClient();
		}
		return (string[])client.Call(app, "DCOPClient", "QCStringList objects()");
	}

	[DcopCall("QCStringList functions()")]
	public string[] functions()
	{
		return (string[])Call("QCStringList functions()");
	}

	[DcopCall("QCStringList interfaces()")]
	public string[] interfaces()
	{
		return (string[])Call("QCStringList interfaces()");
	}

	public string App
	{
		get
		{
			return app;
		}
		set
		{
			if(initialised)
			{
				throw new DcopConnectionException("Attempt to change internal state while already initialised");
			}
			app = value;
		}
	}

	public string Obj
	{
		get
		{
			return obj;
		}
		set
		{
			if(initialised)
			{
				throw new DcopConnectionException("Attempt to change internal state while already initialised");
			}
			obj = value;
		}
	}

/*	public string Type
	{
		get
		{
			return Type;
		}
	}*/

	public DcopClient Client
	{
		get
		{
			return client;
		}
		set
		{
			if(initialised)
			{
				throw new DcopConnectionException("Attempt to change internal state while already initialised");
			}
			client = value;
		}
	}

}; // Class DcopRef

} // Namespace Xsharp.Dcop

